from django.http import HttpResponseRedirect, HttpResponseBadRequest
from django.views.generic.base import View
from flaggit import utils
from flaggit.forms import FlagForm

class FlagView(View):
    def post(self, request):
        form = FlagForm(request.POST)
        
        if not form.is_valid():
            return HttpResponseBadRequest()
        
        user = None
        if request.user.is_authenticated():
            user = request.user
        
        utils.flag(form.cleaned_data['object'], user, request.META['REMOTE_ADDR'],
            form.cleaned_data['comment'])
        
        if 'next' in request.GET:
            return HttpResponseRedirect(request.GET.get('next'))
        return HttpResponseRedirect(request.META.get('HTTP_REFERER', '/'))
        



from django import template
from django.contrib.contenttypes.models import ContentType

register = template.Library()

@register.tag
def flag_form(parser, token):
    """
    Renders the flagging form for a given object. Optionally with a custom 
    template.
    
    Usage:
    
        {% load flaggit_tags %}
        {% flag_form object %}
        {% flag_form object "flaggit/alternate_form.html" %}
    """
    
    bits = token.split_contents()
    
    return FlagFormNode(*bits[1:])

class FlagFormNode(template.Node):
    def __init__(self, obj, tpl=None):
        self.obj = template.Variable(obj)
        self.tpl = tpl[1:-1] if tpl else 'flaggit/form.html'
    
    def render(self, context):
        obj = self.obj.resolve(context)
        content_type = ContentType.objects.get_for_model(obj)
        return template.loader.render_to_string(self.tpl, {
            'object': obj,
            'content_type': content_type,
        }, context_instance=context)



from django.contrib.auth.models import User
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models.signals import post_save, post_init
from flaggit.signals import flagged, review, rejected, approved


FLAGGED = 1
REVIEW = 2
CONTENT_REJECTED = 3
CONTENT_APPROVED = 4

FLAG_CHOICES = (
    (FLAGGED, 'Flagged'),
    (REVIEW, 'Under review'),
    (CONTENT_REJECTED, 'Rejected'),
    (CONTENT_APPROVED, 'Approved'),
)

class Flag(models.Model):
    object_id = models.PositiveIntegerField()
    content_type = models.ForeignKey(ContentType)
    content_object = generic.GenericForeignKey("content_type", "object_id")
    
    _pre_save_status = None
    status = models.PositiveIntegerField(choices=FLAG_CHOICES, default=FLAGGED)
    created = models.DateTimeField(auto_now_add=True)
    reviewed = models.DateTimeField(blank=True, null=True)
    reviewer = models.ForeignKey(User, blank=True, null=True)
    
    comment = models.TextField(blank=True, null=True)

    def __unicode__(self):
        return u'%s: %s' % (self.get_status_display(), self.content_object)


class FlagInstance(models.Model):
    flag = models.ForeignKey(Flag, related_name='flags')
    user = models.ForeignKey(User, blank=True, null=True)
    ip = models.IPAddressField(blank=True, null=True)
    datetime = models.DateTimeField(auto_now_add=True)
    
    comment = models.TextField(blank=True, null=True)
    
    def __unicode__(self):
        return u'%s: %s' % (self.user, self.flag.content_object)

def post_init_handler(sender, instance, **kwargs):
    instance._pre_save_status = instance.status

def flag_handler(sender, instance, created=False, **kwargs):
    if created:
        flagged.send(instance.content_object, flag=instance, created=created)
        return

    # The status didn't change. Return.
    if instance._pre_save_status == instance.status:
        return

    if instance.status == FLAGGED:
        flagged.send(instance.content_object, flag=instance, created=False)
    elif instance.status == REVIEW:
        review.send(instance.content_object, flag=instance)
    elif instance.status == CONTENT_REJECTED:
        rejected.send(instance.content_object, flag=instance)
    elif instance.status == CONTENT_APPROVED:
        approved.send(instance.content_object, flag=instance)

    
def flag_instance_handler(sender, instance, created=False, **kwargs):
    if not created:
        return
    
    flagged.send(instance.flag.content_object, flag=instance.flag, created=False)

post_init.connect(post_init_handler, sender=Flag, dispatch_uid='flaggit.flag.post_init')
post_save.connect(flag_handler, sender=Flag, dispatch_uid='flaggit.flag.post_save')
post_save.connect(flag_instance_handler, sender=FlagInstance,
    dispatch_uid='flaggit.flaginstance.post_save')



from django import forms
from django.contrib.contenttypes.models import ContentType

class FlagForm(forms.Form):
    object_id = forms.IntegerField(widget=forms.HiddenInput)
    content_type = forms.ModelChoiceField(widget=forms.HiddenInput,
        queryset=ContentType.objects.all())
    comment = forms.CharField(widget=forms.Textarea, required=False)
    
    def clean(self):
        content_type = self.cleaned_data['content_type']
        Model = content_type.model_class()
        id = self.cleaned_data['object_id']
        try:
            obj = Model.objects.get(id=id)
        except Model.DoesNotExist:
            raise forms.ValidationError("No such %s object with id %s" % (Model, id))
        
        self.cleaned_data['object'] = obj
        return self.cleaned_data
    



from django.contrib import admin
from flaggit.models import Flag, FlagInstance, CONTENT_APPROVED, \
    CONTENT_REJECTED
from datetime import datetime

class FlagAdmin(admin.ModelAdmin):
    list_filter = ('status',)
    list_display = ('status', 'link', 'created',
        'reviewer', 'reviewed', 'num_flags')
    
    actions = ['approve', 'reject']
    actions_on_bottom = True
    
    def num_flags(self, obj):
        return obj.flags.all().count()
    
    def link(self, obj):
        try:
            return u'<a href="%s">%s</a>' % (
                obj.content_object.get_absolute_url(), obj.content_object)
        except TypeError:
            return 
    
    link.allow_tags = True
    
    def approve(self, request, queryset):
        for obj in queryset:
            obj.status = CONTENT_APPROVED
            obj.reviewer = request.user
            obj.reviewed = datetime.now()
            obj.save()
    approve.short_description = "Approve content on selected flags. (Save content)"
    
    def reject(self, request, queryset):
        for obj in queryset:
            obj.status = CONTENT_REJECTED
            obj.save()
    reject.short_description = "Reject content on selected flags. (Delete content)"
    
    
    def get_actions(self, request):
        actions = super(FlagAdmin, self).get_actions(request)
        del actions['delete_selected']
        return actions

admin.site.register(Flag, FlagAdmin)
admin.site.register(FlagInstance)




from django import template
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.test import TestCase
from flaggit import utils
from flaggit.models import Flag, FlagInstance, REVIEW, CONTENT_APPROVED, \
    CONTENT_REJECTED
from flaggit.signals import flagged, review, rejected, approved


class FlaggingTest(TestCase):
    def setUp(self):
        self.hendrix = User.objects.create(username='hendrix')
        self.lennon = User.objects.create(username='lennon')
        self.hendrix.set_password('guitar')
        self.hendrix.save()
        self.ctype = ContentType.objects.get_for_model(self.hendrix)

        self.url = reverse('flaggit')

    def test_anonymous_flagging(self):
        response = self.client.post(self.url, {
            'content_type': self.ctype.id,
            'object_id': self.lennon.id
        })
        
        self.assertEqual(302, response.status_code)
        
        self.assertEqual(1, Flag.objects.all().count())
        self.assertEqual(self.lennon, Flag.objects.all()[0].content_object)
        self.assertEqual(1, Flag.objects.all()[0].flags.count())
        
    def test_non_dupe_anonymous_flagging(self):
        response = self.client.post(self.url, {
            'content_type': self.ctype.id,
            'object_id': self.lennon.id
        })
        
        self.assertEqual(302, response.status_code)
        
        self.assertEqual(1, Flag.objects.all().count())
        self.assertEqual(self.lennon, Flag.objects.all()[0].content_object)
        self.assertEqual(1, Flag.objects.all()[0].flags.count())
        
        response = self.client.post(self.url, {
            'content_type': self.ctype.id,
            'object_id': self.lennon.id
        })
        
        self.assertEqual(302, response.status_code)
        
        self.assertEqual(1, Flag.objects.all().count())
        self.assertEqual(self.lennon, Flag.objects.all()[0].content_object)
        self.assertEqual(1, Flag.objects.all()[0].flags.count())

    def test_flagging(self):
        self.client.login(username='hendrix', password='guitar')
        
        response = self.client.post(self.url, {
            'content_type': self.ctype.id,
            'object_id': self.lennon.id
        })
        
        self.assertEqual(302, response.status_code)
        
        self.assertEqual(1, Flag.objects.all().count())
        self.assertEqual(self.lennon, Flag.objects.all()[0].content_object)
        self.assertEqual(1, Flag.objects.all()[0].flags.count())
        self.assertEqual(self.hendrix, FlagInstance.objects.all()[0].user)
    
    def test_non_dupe_flagging(self):
        self.client.login(username='hendrix', password='guitar')
        
        response = self.client.post(self.url, {
            'content_type': self.ctype.id,
            'object_id': self.lennon.id
        })
        
        self.assertEqual(302, response.status_code)
        
        self.assertEqual(1, Flag.objects.all().count())
        self.assertEqual(self.lennon, Flag.objects.all()[0].content_object)
        self.assertEqual(1, Flag.objects.all()[0].flags.count())
        self.assertEqual(self.hendrix, FlagInstance.objects.all()[0].user)
        
        response = self.client.post(self.url, {
            'content_type': self.ctype.id,
            'object_id': self.lennon.id
        })
        
        self.assertEqual(302, response.status_code)
        
        self.assertEqual(1, Flag.objects.all().count())
        self.assertEqual(self.lennon, Flag.objects.all()[0].content_object)
        self.assertEqual(1, Flag.objects.all()[0].flags.count())
        self.assertEqual(self.hendrix, FlagInstance.objects.all()[0].user)


    def test_flagging_get(self):
        response = self.client.get(self.url, {
            'content_type': self.ctype.id,
            'object_id': self.lennon.id
        })
        
        self.assertEqual(405, response.status_code)

    def test_flagging_fail(self):
        response = self.client.post(self.url, {
            'content_type': self.ctype.id,
            'object_id': 12345
        })
        
        self.assertEqual(400, response.status_code)

    def test_template_tag(self):
        ctx = template.Context({'obj':self.lennon})
        tpl = template.Template("""{% load flaggit_tags %}{% flag_form obj %}""")

        self.assertTrue(isinstance(tpl.render(ctx), unicode))
        
        tpl = template.Template("""{% load flaggit_tags %}{% flag_form obj "flaggit/form.html" %}""")
        
        self.assertTrue(isinstance(tpl.render(ctx), unicode))


    def test_signals(self):
        Handler = type('Handler', (object,), {
            'inc': lambda self: setattr(self, 'i', getattr(self, 'i') + 1),
            'i': 0
        })
        flagged_handler = Handler()
        review_handler = Handler()
        rejected_handler = Handler()
        approved_handler = Handler()
        
        def handle_flagged(sender, flag, created=False, **kwargs):
            if created:
                flagged_handler.inc()
        
        def handle_review(sender, flag, **kwargs):
            review_handler.inc()
            
        def handle_rejected(sender, flag, **kwargs):
            rejected_handler.inc()
        
        def handle_approved(sender, flag, **kwargs):
            approved_handler.inc()
        
        flagged.connect(handle_flagged)
        review.connect(handle_review)
        rejected.connect(handle_rejected)
        approved.connect(handle_approved)

        utils.flag(self.lennon, user=self.hendrix)
        
        flag = Flag.objects.all()[0]
        
        flag.status = REVIEW
        flag.save()
        flag.status = CONTENT_APPROVED
        flag.save()
        flag.status = CONTENT_REJECTED
        flag.save()
        
        self.assertEqual(1, flagged_handler.i)
        self.assertEqual(1, review_handler.i)
        self.assertEqual(1, rejected_handler.i)
        self.assertEqual(1, approved_handler.i)



#!/usr/bin/env python

from setuptools import setup, find_packages
import flaggit

setup(
    name='django-flaggit',
    description='Generic content flagging for Django',
    long_description=open('README.rst').read(),
    packages=['flaggit', 'flaggit.templatetags'],
    author='Alen Mujezinovic',
    author_email='alen@caffeinehit.com',
    url='https://github.com/caffeinehit/django-flaggit',
    version=flaggit.__version__,
    include_package_data=True,
    package_data={'flaggit': 'templates/flaggit/*html'},
    zip_safe=False,
)



from django.dispatch.dispatcher import Signal

flagged = Signal(providing_args=['flag', 'created'])
review = Signal(providing_args=['flag'])
rejected = Signal(providing_args=['flag'])
approved = Signal(providing_args=['flag'])



from django.contrib.contenttypes.models import ContentType
from flaggit.models import Flag, FlagInstance

def flag(obj, user=None, ip=None, comment=None):
    flag, created = Flag.objects.get_or_create(
        object_id=obj.pk,
        content_type=ContentType.objects.get_for_model(obj)
    )
    
    if user:
        flag_instance, created = FlagInstance.objects.get_or_create(
            flag=flag,
            user=user,
        )
    else:
        flag_instance, created = FlagInstance.objects.get_or_create(
            flag=flag,
            ip=ip
        )
    flag_instance.ip = ip
    flag_instance.comment = comment
    flag_instance.save()
    
    return flag_instance
    
